PROC CArray()   ;Pronounced 'seearray  
;Ŀ
;CArray 1.0 (C)  1993 Data Works of Houston. All rights reserved              
;   Written by: Cary Jensen of Data Works of Houston                          
;               (713) 359-3311                                                
;   Data Works of Houston specializes in Paradox consulting, training, and    
;   application development.                                                  
;                                                                             
;   We can provide all your Paradox and Paradox for Windows solutions.        
;                                                                             
; Description:                                                                
;   CArray displays a dialog box of currrently defined arrays.  Select one of 
;   these arrays to display that array's contents.  After the first time you  
;   use CArray during your Paradox session, you must use Update to update the 
;   list of arrays if you have created or released one or more arrays.        
;                                                                             
;   This utility is offered to Paradox users free of charge.                  
;   You may give copies of CArray to any other Paradox users who have a        
;   use for it as long as these copies have not been modified in any way.     
;                                                                             
;   Variables used:                                                           
;      ArrayName      :name of array to view                                  
;      ArrayNameList  :dynamic array of defined array names (persistent)      
;      CFixedArray    :fixed array holds display array contents               
;      C              :counter variable for indexing CFixedArray              
;      ErrorProc      :declared private to prevent triggering your error proc                                                                      
;      GetNames       :flag identifying that array names have been loaded     
;      z and zz       :flags that identify the array type (z also used as     
;                     :a generic utility variable)                            
;                                                                             
;                      < < < <   DISCLAIMER   > > > > >                       
;   Data Works of Houston does not guarantee the suitability of CArray for    
;   any purpose and cannot be held liable for any damage caused either        
;   directly or indirectly through the use of Carray.  By using CArray you    
;   agree to these terms                                                      
;
PRIVATE ErrorProc, z, zz, CFixedArray, GotNames, C

;Check for compatibility mode or version < 4.0
MENU HOME
IF NOT MENUCHOICE() = CHR(240)
  THEN
    QUIT "CArray requires Paradox 4 or later running in standard mode."
ENDIF

GotNames = False

PROC LoadArrayNames()
;This procedure loads the array names to the array HoldArrayNames
  PRIVATE Count, SaveVarsRestore, HoldSaveVars, CurrentChar, FoundOne
  If GotNames
    THEN
      SELECTCONTROL "ANTag"
      MESSAGE "Displayed names are current."
      RETURN True
  ENDIF
      
  MESSAGE "Loading array names..."
  IF ISFILE("SAVEVARS.SC1")
    THEN RUN NOREFRESH "DEL SAVEVARS.SC1 >NUL"
  ENDIF
  IF ISFILE("SAVEVARS.SC")
    THEN
      SaveVarsRestore = True
      RUN NOREFRESH "REN SAVEVARS.SC SAVEVARS.SC1 >NUL"
    ELSE SaveVarsRestore = False
  ENDIF

  ;Use the SAVEVARS command to write the list of current
  ;variables and arrays to a file.  Next, read this file
  ;to the variable HoldSaveVars.
  SAVEVARS ALL
  FILEREAD "SAVEVARS.SC" TO HoldSaveVars

  ;Search the HoldSaveVars string to determine fixed array names.
  Count = 0
  CurrentChar = 1
  DYNARRAY ArrayNameList[]
  FoundOne = SEARCHFROM(CHR(10)+"Array ",HoldSaveVars,CurrentChar)
  WHILE NOT FoundOne = 0
    ;We got one.  Increment Count, Get position of [, 
    ;clip out array name, and search for next array
    Count = Count + 1
    CurrentChar = SEARCHFROM("[",HoldSaveVars,FoundOne+8)
    ArrayNameList[SUBSTR(HoldSaveVars,FoundOne+7,CurrentChar - (FoundOne+7))]
      = SUBSTR(HoldSaveVars,FoundOne+7,CurrentChar - (FoundOne+7))
    FoundOne = SEARCHFROM(CHR(10)+"Array ",HoldSaveVars,CurrentChar)
  ENDWHILE  

  ;All arrays have been found.  Now look for dynamic arrays.
  CurrentChar = 1
  FoundOne = SEARCHFROM(CHR(10)+"Dynarray ",HoldSaveVars,CurrentChar)
  WHILE NOT FoundOne = 0
    ;We got one.  Increment Count, Get position of [, 
    ;clip out dynarray name, and search for next dynarray
    Count = Count + 1
    CurrentChar = SEARCHFROM("[",HoldSaveVars,FoundOne+11)
    ArrayNameList[SUBSTR(HoldSaveVars,FoundOne+10,CurrentChar - (FoundOne+ 10))] 
      = SUBSTR(HoldSaveVars,FoundOne+10,CurrentChar - (FoundOne+ 10))
    FoundOne = SEARCHFROM(CHR(10)+"Dynarray ",HoldSaveVars,CurrentChar)
  ENDWHILE  
  
  ;Restore the Savevars script, if necessary.
  IF SaveVarsRestore
    THEN
      z = FILESIZE("SAVEVARS.SC")   ;Ensure SAVEVARS.SC is a closed file.
      RUN NOREFRESH "DEL SAVEVARS.SC >NUL"
      RUN NOREFRESH "REN SAVEVARS.SC1 SAVEVARS.SC >NUL"
  ENDIF

  ;Verify at least one array of Dynarray was found
  GotNames = True
  IF Count = 0
    THEN
      SELECTCONTROL "CTag"
      MESSAGE "Sorry, there are no fixed or dynamic arrays to see."
      RETURN False
    ELSE
      REFRESHCONTROL "ANTag"
      SELECTCONTROL "ANTag"
      MESSAGE "Names displayed are current"
      RETURN True  
  ENDIF
ENDPROC ;LoadArrayNames()

PROC HelpScreen()
;This is a dialog proc used to display a help screen.
  SHOWDIALOG "CArray Help"
    @ 4, 13
    HEIGHT 16 WIDTH 52
    @ 1,  1 ?? " Select an array to view from displayed list"
    @ 2,  1 ?? "  then select the OK button."
    @ 4,  1 ?? " Select Cancel to exit."
    @ 6,  1 ?? " Select Update to update the list of available"
    @ 7,  1 ?? "  arrays. You should update the list if the"
    @ 8,  1 ?? "  array you want to display is not listed, or"
    @ 9,  1 ?? "  you have added or deleted arrays. You do not"
    @ 10, 1 ?? "  have to update if array contents have changed."

    PUSHBUTTON @ 12, 19 WIDTH 10
      "~O~K" OK DEFAULT
      VALUE "True" TAG "OTag" TO RetVal
  ENDDIALOG
  RETURN True
ENDPROC ; HelpScreen()

;Begin Main
IF NOT ISASSIGNED(ArrayNameList)
  THEN
    LoadArrayNames()
    IF NOT Retval
      THEN
        GETEVENT TO z
        RELEASE VARS z
        QUIT
    ENDIF
ENDIF

;Display the array list.
SHOWDIALOG "Select the array to view"
  @ 5,9 HEIGHT 18 WIDTH 60
  @ 1,3 ?? "Array name: "
  PICKDYNARRAY @ 1,16 HEIGHT 10 WIDTH 40
    ArrayNameList TAG "ANTag" TO ArrayName
  PUSHBUTTON @ 13, 15 WIDTH 10 "~O~k" OK DEFAULT
    VALUE True TAG "OTag" TO Retval
  PUSHBUTTON @ 13, 25 WIDTH 10 "~C~ancel" CANCEL 
    VALUE False Tag "CTag" TO Retval
  PUSHBUTTON @ 13, 35 WIDTH 10 "~U~pdate"
    VALUE LoadArrayNames() TAG "CTag" TO Retval
  PUSHBUTTON @ 13, 45 WIDTH 10 "~H~elp"
    VALUE HelpScreen() TAG "HTag" TO Retval
ENDDIALOG

IF NOT Retval OR ISBLANK(ArrayName)
  THEN QUIT "CArray canceled."
ENDIF

;Determine the array type.  This works because ARRAYSIZE and
;DYNARRAYSIZE return 0 if the array name is not of the correct type.
Command = "z = ARRAYSIZE("+ArrayName+") zz = DYNARRAYSIZE("+ArrayName+")"
EXECUTE Command
IF NOT (zz > 0 OR z > 0)
  THEN
    QUIT ArrayName +" is not an array or has no elements."
ENDIF

;Create CFixedArray array to use for display
ARRAY CFixedArray[MAX(z,zz)]
MESSAGE "One moment please..."
IF z > 0
  THEN
      ;Assign the index and element contents of the selected
      ;fixed array to the elements of the array CFixedArray.
      EXECUTE "FOR c from 1 to z "+
        "CFixedArray[c] = FORMAT(\"W8,AL\",c)"+
        "+IIF(ISASSIGNED("+ArrayName+"[c]),"+
             ArrayName+
             "[c],\"(blank)\") ENDFOR"
  ELSE
    ;Assign the index and element contents of the selected
    ;dynamic array to the elements of the array CFixedArray.
    C = 1
    EXECUTE "FOREACH i in "+ArrayName +
       " CFixedArray[c] = FORMAT(\"W20,AL\",i)"+
        "+IIF(ISASSIGNED("+ArrayName+"[i]),strval("+
             ArrayName+
             "[i]),\"(blank)\") c = c + 1 ENDFOREACH"
ENDIF
MESSAGE ""

;Display the Viewing dialog box.
SHOWDIALOG "Viewing "+  IIF(z=0,"dynamic","fixed")+
           " array "+ArrayName
  @ 2,9 height 20 width 60
  @ 14,1 ?? ArrayName+" has "+STRVAL(MAX(z,zz))+" elements"
  @ 0, 2 ?? "Index"+IIF(z=0,SPACES(15),SPACES(3))+"Contents"
  PICKARRAY
    @ 1,1 height 12 width 55
    CFixedArray
    TAG "ArrTag" TO RetVal
  PUSHBUTTON 
    @ 16,23 WIDTH 10 
    "~D~one"
    OK DEFAULT 
    Value "T" TAG "P" TO RetVal
ENDDIALOG

ENDPROC
;This next line executes the procedure CArray()
CARRAY()
